export(AmpPhaseDecomp,
       as.array3,
       as.fd,
       as.POSIXct1970,
       axisIntervals, 
       axesIntervals,
       basisfd,
       bifd,
       bifdPar,
       boxplot.fd, 
       boxplot.fdPar, 
       boxplot.fdSmooth,
       bsplinepen,
       bsplineS,
       cca.fd,
       center.fd,
       checkDim3, 
       checkDims3,
       checkLogical, 
       checkNumeric, 
       checkLogicalInteger,
       cor.fd,
       covPACE,
       CRAN,
       create.bspline.basis, 
       create.constant.basis,
       create.exponential.basis,
       create.fourier.basis,
       create.monomial.basis,
       create.polygonal.basis,
       create.power.basis,
       CSTR2in, 
       CSTR2, 
       CSTRfitLS, 
       CSTRfn, 
       CSTRres, 
       CSTRsse,
       cumfd,
       cycleplot.fd,
       Data2fd,
       deriv.fd,
       df2lambda,
       dirs,
       Eigen,
       eigen.pda,
       eval.basis,
       eval.bifd,
       eval.fd,
       eval.monfd,
       eval.penalty,
       eval.posfd,
       evaldiag.bifd,
       expon,
       exponentiate.fd,
       exponpen,
       fd, fbplot,
       fd2list,
       fdlabels,
       fdPar,
       fdParcheck,
       fourier,
       fourierpen,
       Fperm.fd,
       fRegress,
       fRegress.fd,
       fRegress.CV,
       fRegress.stderr,
       fRegress.double,
       fRegress.formula,
       Fstat.fd,
       geigen,
       getbasismatrix,
       getbasispenalty,
       getbasisrange,
       inprod,
       inprod.bspline,
       int2Lfd,
       intensity.fd,
       is.basis,
       is.fd,
       is.fdPar,
       is.fdSmooth,
       is.Lfd,
       lambda2df,
       lambda2gcv,
       landmarkreg,
       Lfd,
       lines.fdSmooth,
       linmod,
       lnsrch,
       matplot, 
       matplot.default,
       mean.fd,
       minus.fd, 
       monfn,
       mongrad,
       monhess,
       monomial,
       monomialpen,
       norder, 
       norder.bspline,
       objAndNames,
       odesolv,
       pca.fd,
       pcaPACE,
       pda.fd,
       pda.overlay,
       phaseplanePlot,
       polintmat,
       plot.cca.fd,
       plot.fd,
       plot.pca.fd,
       plotbeta,
       plotfit.fd, 
       plotfit.fdSmooth,
       plotscores,
       plus.fd, 
       polyg,
       polygpen,
       powerbasis,
       powerpen,
       ppBspline,
       predict.basisfd, 
       predict.fd, 
       predict.fdPar, 
       predict.fRegress, 
       predict.fdSmooth, 
       project.basis,
       quadset,
       reconsCurves,
       register.fd,
       register.newfd,
       scoresPACE,
       sd.fd, 
       std.fd, 
       stdev.fd, 
       stddev.fd,
       smooth.basis,
       smooth.basis1,
       smooth.basis2,
       smooth.basis3,
       smooth.bibasis,
       smooth.fd,
       smooth.basisPar,
       smooth.fdPar,
       smooth.monotone,
       smooth.morph,
       smooth.pos,
       smooth.basis.sparse,
       smooth.sparse.mean,
       sparse.list,
       sparse.mat,
       stepchk,
       stepit,
       symsolve,
       "[.fd",
       times.fd,
       tperm.fd,
       trapzmat,
       var.fd,
       varmx,
       varmx.cca.fd,
       varmx.pca.fd,
       vec2Lfd, 
       wtcheck,
       ycheck,
       zerobasis,
       zerofind)

import("graphics")
import("stats")
import("splines")
import("fds")
import("deSolve")

importFrom("grDevices", "dev.new", "heat.colors")
importFrom("utils", "data")

S3method("==",         basisfd)
S3method('*',          basisfd)
S3method(summary,      basisfd)
S3method(norder,       basisfd)
S3method(plot,         basisfd)
S3method(predict,      basisfd)
S3method('+',          fd)
S3method('-',          fd)
S3method('*',          fd)
S3method('[',          fd)
S3method('^',          fd)
S3method(boxplot,      fd)
S3method(coef,         fd)
S3method(coefficients, fd)
S3method(lines,        fd)
S3method(norder,       fd)
S3method(plot,         fd)
S3method(predict,      fd)
S3method(sqrt,         fd)
S3method(sum,          fd)
S3method(summary,      fd)
S3method(fRegress,     fd)
S3method(boxplot,      fdPar)
S3method(coef,         fdPar)
S3method(coefficients, fdPar)
S3method(plot,         fdPar)
S3method(predict,      fdPar)
S3method(summary,      fdPar)
S3method(as.fd,        fdSmooth)
S3method(coef,         fdSmooth)
S3method(boxplot,      fdSmooth)
S3method(coefficients, fdSmooth)
S3method(fitted,       fdSmooth)
S3method(lines,        fdSmooth)
S3method(residuals,    fdSmooth)
S3method(plot,         fdSmooth)
S3method(predict,      fdSmooth)
S3method(predict,      fRegress)
S3method(plot,         Lfd)
S3method(summary,      Lfd)
S3method(fitted,       monfd)
S3method(residuals,    monfd)
S3method(predict,      monfd)
S3method(as.fd,        'function')
S3method(as.fd,        smooth.spline)
S3method(fRegress,     double)
S3method(fRegress,     formula)
S3method(fRegress,     character)
S3method(fRegress,     stderr)
S3method(fRegress,     CV)
S3method(fitted,       posfd)
S3method(predict,      posfd)
S3method(residuals,    posfd)
S3method(matplot,      Date)
S3method(matplot,      default)
S3method(matplot,      POSIXct)
S3method(norder,       bspline)
S3method(norder,       default)
S3method(plot,         cca.fd)
S3method(plot,         pca.fd)
S3method(plot,         pda.fd)
S3method(summary,      bifd)
S3method(matplot,      matrix)
S3method(matplot,      numeric)

